#include <string>
#include <iostream>
#include <sstream>

#include <sys/types.h>
#include <sys/socket.h>
#include <ifaddrs.h>
#include <arpa/inet.h>


#include <omniORB4/CORBA.h>

#include "OrbWrapper.hh"

/**
 * Get the ip address for a given network interface name.
 * @return The ip address for a given network interface name or an empty
 *         string on error.
 */
std::string getIPAddressForInterface(const std::string& interface) {
    std::string addr;
    if (interface.empty()) return addr;
    struct ifaddrs* ifa_ptr;
    struct ifaddrs* ifap;
    const char *ifStr = interface.c_str();
    int ret = getifaddrs(&ifa_ptr);
    if (ret != 0) return addr;
    ifap = ifa_ptr;
    while (ifap != 0) {
        if (strcmp(ifap->ifa_name, ifStr) == 0) {
            // Interface found
            if (ifap->ifa_addr->sa_family == AF_INET) {
                // Address is an ipv4 address -> convert to string
                const char *ret;
                char buf[INET_ADDRSTRLEN];
                struct sockaddr_in* soap =
                    reinterpret_cast<struct sockaddr_in*>(ifap->ifa_addr);
                ret = inet_ntop(AF_INET,
                    reinterpret_cast<const void*>(&(soap->sin_addr)),
                    buf, INET_ADDRSTRLEN);
                freeifaddrs(ifa_ptr);
                if (ret == 0) return addr;
                else return buf;
            } else if (ifap->ifa_addr->sa_family == AF_INET6) {
                // Address is an ipv6 address -> convert to string
                const char *ret;
                char buf6[INET6_ADDRSTRLEN];
                struct sockaddr_in* soap =
                    reinterpret_cast<struct sockaddr_in*>(ifap->ifa_addr);
                ret = inet_ntop(AF_INET6,
                    reinterpret_cast<const void*>(&(soap->sin_addr)),
                    buf6, INET6_ADDRSTRLEN);
                freeifaddrs(ifa_ptr);
                if (ret == 0) return addr;
                else return buf6;
            }
        }
        ifap = ifap->ifa_next;
    }
    freeifaddrs(ifa_ptr);
    return addr;
}

int main(int argc, char** argv) {
    int basePort = 50000;
    int maxTries = 1;
    std::string addr = getIPAddressForInterface("eth0");
    if (addr.empty() && argc != 2) {
        std::cout << "Could not get ip address for interface eth0."
            << " Please specify on the command line.";
        return 1;
    }
    if (addr.empty() && argc == 2) {
        addr = argv[1];
    }
    std::string nameserver = "corbaloc:iiop:1.2@localhost:12345/TNameService";
    unsigned int outConScanPeriod = 0;
    unsigned int inConScanPeriod = 0;
    unsigned int scanGranularity = 0;
    unsigned int clientCallTimeOutPeriod = 0;
    unsigned int clientConnectTimeOutPeriod = 0;

    // Try to initialize ORB on free port
    OrbWrapper* orbWrapper = 0;
    for (int i = 0; i < maxTries; i++) {
        int port = basePort + i;
        std::ostringstream endpoint;
        endpoint << "giop:tcp:" << addr << ":" << port;

        orbWrapper = OrbWrapper::createInstance(
            nameserver.c_str(),
            endpoint.str().c_str(),
            outConScanPeriod,
            inConScanPeriod,
            scanGranularity,
            clientCallTimeOutPeriod,
            clientConnectTimeOutPeriod);

        if (orbWrapper != 0) {
            break;
        }
    }

    if (orbWrapper == 0) {
        std::cerr << "Orb initialization failed." << std::endl;
        return 1;
    }

    // We have an ORB and an activated root poa
    {
        CORBA::ORB_var orb = orbWrapper->getORB();
        PortableServer::POA_var poa = orbWrapper->getRootPOA();
        sleep(20);
    }

    delete orbWrapper;
}

